@@ -1,8 +1,5 @@
-#!/usr/bin/env python
-# -*- encoding: utf-8 -*-
-
 # Copyright (c) "Neo4j"
-# Neo4j Sweden AB [http://neo4j.com]
+# Neo4j Sweden AB [https://neo4j.com]
 #
 # This file is part of Neo4j.
 #
@@ -10,7 +7,7 @@
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#     http://www.apache.org/licenses/LICENSE-2.0
+#     https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -19,24 +16,43 @@
 # limitations under the License.
 
 
+from __future__ import annotations
+
+import logging
+import typing as t
 from socket import (
-    getaddrinfo,
-    getservbyname,
-    SOCK_STREAM,
+    AddressFamily,
     AF_INET,
     AF_INET6,
+    getservbyname,
 )
-import logging
+
+
+if t.TYPE_CHECKING:
+    import typing_extensions as te
 
 
 log = logging.getLogger("neo4j")
 
 
-class _AddressMeta(type(tuple)):
+_T = t.TypeVar("_T")
 
-    def __init__(self, *args, **kwargs):
-        self._ipv4_cls = None
-        self._ipv6_cls = None
+
+if t.TYPE_CHECKING:
+
+    class _WithPeerName(te.Protocol):
+        def getpeername(self) -> tuple: ...
+
+
+assert type(tuple) is type
+
+
+class _AddressMeta(type):
+
+    def __init__(cls, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        cls._ipv4_cls = None
+        cls._ipv6_cls = None
 
     def _subclass_by_family(self, family):
         subclasses = [
@@ -68,16 +84,25 @@ class _AddressMeta(type(tuple)):
 class Address(tuple, metaclass=_AddressMeta):
 
     @classmethod
-    def from_socket(cls, socket):
+    def from_socket(
+        cls: t.Type[_T_Address],
+        socket: _WithPeerName
+    ) -> _T_Address:
         address = socket.getpeername()
         return cls(address)
 
     @classmethod
-    def parse(cls, s, default_host=None, default_port=None):
+    def parse(
+        cls: t.Type[_T_Address],
+        s: str,
+        default_host: str = None,
+        default_port: int = None
+    ) -> _T_Address:
         if not isinstance(s, str):
             raise TypeError("Address.parse requires a string argument")
         if s.startswith("["):
             # IPv6
+            port: t.Union[str, int]
             host, _, port = s[1:].rpartition("]")
             port = port.lstrip(":")
             try:
@@ -97,16 +122,21 @@ class Address(tuple, metaclass=_AddressMeta):
                         port or default_port or 0))
 
     @classmethod
-    def parse_list(cls, *s, default_host=None, default_port=None):
+    def parse_list(
+        cls: t.Type[_T_Address],
+        *s: str,
+        default_host: str = None,
+        default_port: int = None
+    ) -> t.List[_T_Address]:
         """ Parse a string containing one or more socket addresses, each
         separated by whitespace.
         """
         if not all(isinstance(s0, str) for s0 in s):
             raise TypeError("Address.parse_list requires a string argument")
-        return [Address.parse(a, default_host, default_port)
+        return [cls.parse(a, default_host, default_port)
                 for a in " ".join(s).split()]
 
-    def __new__(cls, iterable):
+    def __new__(cls, iterable: t.Collection) -> Address:
         if isinstance(iterable, cls):
             return iterable
         n_parts = len(iterable)
@@ -121,82 +151,29 @@ class Address(tuple, metaclass=_AddressMeta):
         return inst
 
     #: Address family (AF_INET or AF_INET6)
-    family = None
+    family: t.Optional[AddressFamily] = None
 
     def __repr__(self):
         return "{}({!r})".format(self.__class__.__name__, tuple(self))
 
     @property
-    def host_name(self):
+    def host_name(self) -> str:
         return self[0]
 
     @property
-    def host(self):
+    def host(self) -> str:
         return self[0]
 
     @property
-    def port(self):
+    def port(self) -> int:
         return self[1]
 
     @property
-    def unresolved(self):
+    def unresolved(self) -> Address:
         return self
 
-    @classmethod
-    def _dns_resolve(cls, address, family=0):
-        """ Regular DNS resolver. Takes an address object and optional
-        address family for filtering.
-
-        :param address:
-        :param family:
-        :return:
-        """
-        try:
-            info = getaddrinfo(address.host, address.port, family, SOCK_STREAM)
-        except OSError:
-            raise ValueError("Cannot resolve address {}".format(address))
-        else:
-            resolved = []
-            for fam, _, _, _, addr in info:
-                if fam == AF_INET6 and addr[3] != 0:
-                    # skip any IPv6 addresses with a non-zero scope id
-                    # as these appear to cause problems on some platforms
-                    continue
-                if addr not in resolved:
-                    resolved.append(ResolvedAddress(
-                        addr, host_name=address.host_name)
-                    )
-            return resolved
-
-    def resolve(self, family=0, resolver=None):
-        """ Carry out domain name resolution on this Address object.
-
-        If a resolver function is supplied, and is callable, this is
-        called first, with this object as its argument. This may yield
-        multiple output addresses, which are chained into a subsequent
-        regular DNS resolution call. If no resolver function is passed,
-        the DNS resolution is carried out on the original Address
-        object.
-
-        This function returns a list of resolved Address objects.
-
-        :param family: optional address family to filter resolved
-                       addresses by (e.g. AF_INET6)
-        :param resolver: optional customer resolver function to be
-                         called before regular DNS resolution
-        """
-
-        log.debug("[#0000]  C: <RESOLVE> %s", self)
-        resolved = []
-        if resolver:
-            for address in map(Address, resolver(self)):
-                resolved.extend(self._dns_resolve(address, family))
-        else:
-            resolved.extend(self._dns_resolve(self, family))
-        return resolved
-
     @property
-    def port_number(self):
+    def port_number(self) -> int:
         try:
             return getservbyname(self[1])
         except (OSError, TypeError):
@@ -208,11 +185,14 @@ class Address(tuple, metaclass=_AddressMeta):
                 raise type(e)("Unknown port value %r" % self[1])
 
 
+_T_Address = t.TypeVar("_T_Address", bound=Address)
+
+
 class IPv4Address(Address):
 
     family = AF_INET
 
-    def __str__(self):
+    def __str__(self) -> str:
         return "{}:{}".format(*self)
 
 
@@ -220,25 +200,25 @@ class IPv6Address(Address):
 
     family = AF_INET6
 
-    def __str__(self):
+    def __str__(self) -> str:
         return "[{}]:{}".format(*self)
 
 
 class ResolvedAddress(Address):
 
+    _host_name: str
+
     @property
-    def host_name(self):
+    def host_name(self) -> str:
         return self._host_name
 
     @property
-    def unresolved(self):
+    def unresolved(self) -> Address:
         return super().__new__(Address, (self._host_name, *self[1:]))
 
-    def resolve(self, family=0, resolver=None):
-        return [self]
-
-    def __new__(cls, iterable, host_name=None):
+    def __new__(cls, iterable, *, host_name: str) -> ResolvedAddress:
         new = super().__new__(cls, iterable)
+        new = t.cast(ResolvedAddress, new)
         new._host_name = host_name
         return new
 
